import React from 'react';
import styles from '../styles/LoginDialog.module.css';
import icon_close from '../../public/images/icon-close.svg';
import {Input} from "@nextui-org/input";
import Image from "next/image";
import toast from "react-hot-toast";
import {DateValue} from "@nextui-org/react";
import DatePicker, { registerLocale, setDefaultLocale } from "react-datepicker";
import "react-datepicker/dist/react-datepicker.css";
import { ko } from 'date-fns/locale/ko';
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {faPhotoFilm} from "@fortawesome/free-solid-svg-icons";


interface CreateEventDialogProps {
    onClose: () => void;
}

const CreateEventDialog: React.FC<CreateEventDialogProps> = ({onClose}) => {
    const [closing, setClosing] = React.useState(false);

    const [title, setTitle] = React.useState<string>('');
    const [summary, setSummary] = React.useState<string>('');
    const [description, setDescription] = React.useState<string>('');
    const [prize, setPrize] = React.useState<string>('');
    const [startDate, setStartDate] = React.useState<any>(null);
    const [dueDate, setDueDate] = React.useState<any>(null);
    const [location, setLocation] = React.useState<string>('');

    const [selectedThumbnail, setSelectedThumbnail] = React.useState<string | ArrayBuffer | null>(null);
    const [selectedThumbnailFile, setSelectedThumbnailFile] = React.useState<File | null>(null);

    const [selectedHint1, setSelectedHint1] = React.useState<string | ArrayBuffer | null>(null);
    const [selectedHint1File, setSelectedHint1File] = React.useState<File | null>(null);
    const [selectedHint2, setSelectedHint2] = React.useState<string | ArrayBuffer | null>(null);
    const [selectedHint2File, setSelectedHint2File] = React.useState<File | null>(null);
    const [selectedHint3, setSelectedHint3] = React.useState<string | ArrayBuffer | null>(null);
    const [selectedHint3File, setSelectedHint3File] = React.useState<File | null>(null);


    const [uploading, setUploading] = React.useState<boolean>(false);

    React.useEffect(() => {
        registerLocale('ko', ko);
    }, []);

    React.useEffect(() => {
        const handleKeyDown = (event: KeyboardEvent) => {
            if (event.key === 'Escape') {
                handleClose();
            }
        };

        window.addEventListener('keydown', handleKeyDown);

        return () => {
            window.removeEventListener('keydown', handleKeyDown);
        };
    }, [onClose]);

    const handleClose = () => {
        setClosing(true);
        setTimeout(() => {
            onClose();
        }, 200); // 애니메이션 시간 (0.3s) 후 onClose 실행
    };

    const executeCreateEvent = async () => {
        function toISO8601WithTimezone(date: Date): string {
            const year = date.getFullYear();
            const month = ('0' + (date.getMonth() + 1)).slice(-2);
            const day = ('0' + date.getDate()).slice(-2);
            const hours = ('0' + date.getHours()).slice(-2);
            const minutes = ('0' + date.getMinutes()).slice(-2);
            const seconds = ('0' + date.getSeconds()).slice(-2);
            const timezoneOffset = -date.getTimezoneOffset();
            const sign = timezoneOffset >= 0 ? '+' : '-';
            const offsetHours = ('0' + Math.floor(Math.abs(timezoneOffset) / 60)).slice(-2);
            const offsetMinutes = ('0' + (Math.abs(timezoneOffset) % 60)).slice(-2);

            return `${year}-${month}-${day}T${hours}:${minutes}:${seconds}${sign}${offsetHours}:${offsetMinutes}`;
        }

        const formData = new FormData();
        formData.append('title', title);
        formData.append('subtitle', summary);
        formData.append('description', description);
        formData.append('prize', prize);
        formData.append('start_date', toISO8601WithTimezone(startDate));
        formData.append('due_date', toISO8601WithTimezone(dueDate));
        formData.append('location', location);

        formData.append('thumbnail', selectedThumbnailFile as File);
        if (selectedHint1File) {
            formData.append('hint_image_1', selectedHint1File as File);
        }
        if (selectedHint2File) {
            formData.append('hint_image_2', selectedHint2File as File);
        }
        if (selectedHint3File) {
            formData.append('hint_image_3', selectedHint3File as File);
        }

        const res = fetch('https://rd21.cafe24.com:4343/events/', {
            method: 'POST',
            headers: {
                'Authorization': `Token ${localStorage.getItem('token')}`
            },
            body: formData
        })
            .then(async (res) => {
                if (res.status === 201) {
                    handleClose();
                } else if (res.status === 401) {
                    throw new Error("올바르지 않은 요청이에요")
                } else {
                    throw new Error("예기치 못한 오류에요. 나중에 다시 시도하세요")
                }
            });

        toast.promise(res, {
            loading: "이벤트 게시 중",
            success: '이벤트 게시 완료!',
            error: (err) => err.message
        })

    }

    const handleImageUpload = (event: React.ChangeEvent<HTMLInputElement>, type: string) => {
        const file = event.target.files?.[0];

        if (type === 'thumbnail') {
            if (file) {
                setSelectedThumbnailFile(file);
                const reader = new FileReader();
                reader.onloadend = () => {
                    setSelectedThumbnail(reader.result);
                };
                reader.readAsDataURL(file);
            }
        } else if (type === 'hint1') {
            if (file) {
                setSelectedHint1File(file);
                const reader = new FileReader();
                reader.onloadend = () => {
                    setSelectedHint1(reader.result);
                };
                reader.readAsDataURL(file);
            }
        } else if (type === 'hint2') {
            if (file) {
                setSelectedHint2File(file);
                const reader = new FileReader();
                reader.onloadend = () => {
                    setSelectedHint2(reader.result);
                };
                reader.readAsDataURL(file);
            }
        } else if (type === 'hint3') {
            if (file) {
                setSelectedHint3File(file);
                const reader = new FileReader();
                reader.onloadend = () => {
                    setSelectedHint3(reader.result);
                };
                reader.readAsDataURL(file);
            }
        }

    };



    return (
        <div className={`${styles.overlay} ${closing ? styles.overlayHide : ''}`}
             style={{zIndex: 60}}
        >
            <div className={`${styles.loginDialogContainer} ${closing ? styles.slideDown : ''}`}>
                <div style={{display: 'flex', alignItems: 'center'}}>
                    <p
                        style={{margin: 0, flex: 1}}
                        className={[styles.largeFont, styles.fontBold].join(" ")}
                    >
                        이벤트를 새로 시작해볼까요?
                    </p>
                    <Image src={icon_close} width={20} height={20} alt={'닫기'} onClick={handleClose} />
                </div>


                <div className={styles.inputGroup}>
                    <input id="title" name="title" value={title} onChange={(evt) => setTitle(evt.target.value)}
                           required placeholder="이벤트 제목" style={{marginBottom: '10px'}}
                    />
                    <input id="summary" name="summary" value={summary} onChange={(evt) => setSummary(evt.target.value)}
                           required placeholder="이벤트 한 줄 요약" style={{marginBottom: '10px'}}
                    />
                    <textarea id="description" name="description" value={description} onChange={(evt) => setDescription(evt.target.value)}
                           required placeholder="이벤트 소개" style={{marginBottom: '10px'}}
                    />
                    <input id="prize" name="prize" value={prize} onChange={(evt) => setPrize(evt.target.value)}
                           required placeholder="상품" style={{marginBottom: '10px'}}
                    />
                    <div style={{marginBottom: '10px', display: 'flex', flexDirection: 'column', gap: '10px', width: '100%'}}>
                        <DatePicker
                            placeholderText={'시작 날짜 시간'}
                            selected={startDate}
                            showTimeSelect
                            dateFormat="Pp"
                            timeFormat={"HH:mm"}
                            locale={'ko'}
                            onChange={(date) => {
                                setStartDate(date);
                            }}
                        />
                        <DatePicker
                            placeholderText={'완료 날짜 시간'}
                            selected={dueDate}
                            showTimeSelect
                            dateFormat="Pp"
                            timeFormat={"HH:mm"}
                            locale={'ko'}
                            onChange={(date) => {
                                setDueDate(date);
                            }}
                        />
                    </div>

                    <p
                        style={{margin: 0, flex: 1, marginTop: '20px', marginBottom: '10px'}}
                        className={[styles.normalFont, styles.fontBold, styles.primaryColor].join(" ")}
                    >
                        이벤트 썸네일
                    </p>

                    <div style={{
                        width: '100%',
                        flex: 1,
                    }}>
                        <input
                            type="file"
                            id="thumbnailInput"
                            style={{ display: 'none' }}
                            accept="image/*"
                            onChange={(evt) => handleImageUpload(evt, 'thumbnail')}
                        />
                        {
                            selectedThumbnail ? (
                                <img src={selectedThumbnail as string} alt={'썸네일 이미지'} onClick={() => document.getElementById('thumbnailInput')?.click()}
                                     style={{width: '100%', height: '200px', objectFit: 'cover', borderRadius: '15px'}}
                                />
                            ) : (
                                <div style={{
                                    width: '100%', height: '200px', borderRadius: '15px', backgroundColor: '#c7c7c7',
                                    display: 'flex', flexDirection: 'column', justifyContent: 'center', alignItems: 'center', gap: '10px'
                                }}
                                     onClick={() => document.getElementById('thumbnailInput')?.click()}
                                >
                                    <FontAwesomeIcon icon={faPhotoFilm} />
                                    <p style={{margin: 0, padding: 0}} className={styles.smallFont}>
                                        썸네일 사진 업로드
                                    </p>
                                </div>
                            )}
                    </div>

                    <p
                        style={{margin: 0, flex: 1, marginTop: '20px', marginBottom: '10px'}}
                        className={[styles.normalFont, styles.fontBold, styles.primaryColor].join(" ")}
                    >
                        첫 번째 힌트
                    </p>

                    <div style={{
                        width: '100%',
                        flex: 1,
                    }}>
                        <input
                            type="file"
                            id="hint1Input"
                            style={{ display: 'none' }}
                            accept="image/*"
                            onChange={(evt) => handleImageUpload(evt, 'hint1')}
                        />
                        {
                            selectedHint1 ? (
                                <img src={selectedHint1 as string} alt={'썸네일 이미지'} onClick={() => document.getElementById('hint1Input')?.click()}
                                     style={{width: '100%', height: '200px', objectFit: 'cover', borderRadius: '15px'}}
                                />
                            ) : (
                                <div style={{
                                    width: '100%', height: '200px', borderRadius: '15px', backgroundColor: '#c7c7c7',
                                    display: 'flex', flexDirection: 'column', justifyContent: 'center', alignItems: 'center', gap: '10px'
                                }}
                                     onClick={() => document.getElementById('hint1Input')?.click()}
                                >
                                    <FontAwesomeIcon icon={faPhotoFilm} />
                                    <p style={{margin: 0, padding: 0}} className={styles.smallFont}>
                                        첫 번째 힌트 사진 업로드
                                    </p>
                                </div>
                            )}
                    </div>

                    <p
                        style={{margin: 0, flex: 1, marginTop: '20px', marginBottom: '10px'}}
                        className={[styles.normalFont, styles.fontBold, styles.primaryColor].join(" ")}
                    >
                        두 번째 힌트
                    </p>

                    <div style={{
                        width: '100%',
                        flex: 1,
                    }}>
                        <input
                            type="file"
                            id="hint2Input"
                            style={{ display: 'none' }}
                            accept="image/*"
                            onChange={(evt) => handleImageUpload(evt, 'hint2')}
                        />
                        {
                            selectedHint2 ? (
                                <img src={selectedHint2 as string} alt={'썸네일 이미지'} onClick={() => document.getElementById('hint2Input')?.click()}
                                     style={{width: '100%', height: '200px', objectFit: 'cover', borderRadius: '15px'}}
                                />
                            ) : (
                                <div style={{
                                    width: '100%', height: '200px', borderRadius: '15px', backgroundColor: '#c7c7c7',
                                    display: 'flex', flexDirection: 'column', justifyContent: 'center', alignItems: 'center', gap: '10px'
                                }}
                                     onClick={() => document.getElementById('hint2Input')?.click()}
                                >
                                    <FontAwesomeIcon icon={faPhotoFilm} />
                                    <p style={{margin: 0, padding: 0}} className={styles.smallFont}>
                                        두 번째 힌트 사진 업로드
                                    </p>
                                </div>
                            )}
                    </div>

                    <p
                        style={{margin: 0, flex: 1, marginTop: '20px', marginBottom: '10px'}}
                        className={[styles.normalFont, styles.fontBold, styles.primaryColor].join(" ")}
                    >
                        세 번째 힌트
                    </p>

                    <div style={{
                        width: '100%',
                        flex: 1,
                    }}>
                        <input
                            type="file"
                            id="hint3Input"
                            style={{ display: 'none' }}
                            accept="image/*"
                            onChange={(evt) => handleImageUpload(evt, 'hint3')}
                        />
                        {
                            selectedHint3 ? (
                                <img src={selectedHint3 as string} alt={'썸네일 이미지'} onClick={() => document.getElementById('hint3Input')?.click()}
                                     style={{width: '100%', height: '200px', objectFit: 'cover', borderRadius: '15px'}}
                                />
                            ) : (
                                <div style={{
                                    width: '100%', height: '200px', borderRadius: '15px', backgroundColor: '#c7c7c7',
                                    display: 'flex', flexDirection: 'column', justifyContent: 'center', alignItems: 'center', gap: '10px'
                                }}
                                     onClick={() => document.getElementById('hint3Input')?.click()}
                                >
                                    <FontAwesomeIcon icon={faPhotoFilm} />
                                    <p style={{margin: 0, padding: 0}} className={styles.smallFont}>
                                        세 번째 힌트 사진 업로드
                                    </p>
                                </div>
                            )}
                    </div>

                </div>

                <div style={{
                    display: 'flex', alignItems: 'center',
                    gap: '15px'
                }}>
                    <button className={styles.button} onClick={executeCreateEvent}>
                        게시하기
                    </button>
                </div>


            </div>
        </div>
    );
};

export default CreateEventDialog;
