import React from 'react';
import styles from '../styles/LoginDialog.module.css';
import {Input} from "@nextui-org/input";
import SignupDialog from "@/components/SignupDialog";
import {preventDefaultEvent} from "@material/list/events";
import Image from "next/image";
import icon_close from "../../public/images/icon-close.svg";
import toast, { Toaster } from 'react-hot-toast';
import {exec} from "child_process";


interface LoginDialogProps {
    onClose: () => void;
    authState: React.ComponentState;
    setAuthState: (state: boolean) => void;
}

const LoginDialog: React.FC<LoginDialogProps> = ({onClose, authState, setAuthState}) => {
    const [closing, setClosing] = React.useState(false);
    const [showSignupDialog, setShowSignupDialog] = React.useState<boolean>(false);
    const [email, setEmail] = React.useState<string>('');
    const [password, setPassword] = React.useState<string>('');

    React.useEffect(() => {
        const handleKeyDown = (event: KeyboardEvent) => {
            if (event.key === 'Escape') {
                handleClose();
            }
        };

        window.addEventListener('keydown', handleKeyDown);

        return () => {
            window.removeEventListener('keydown', handleKeyDown);
        };
    }, [onClose]);

    React.useEffect(() => {
        if (authState) {
            toast.error('이미 로그인되어 있습니다');
            handleClose()
        }
    }, []);

    const handleClose = () => {
        setClosing(true);
        setTimeout(() => {
            onClose();
        }, 200); // 애니메이션 시간 (0.3s) 후 onClose 실행
    };

    const executeLogin = async () => {
        const res = fetch('https://rd21.cafe24.com:4343/user/login/', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ email, password })
        })
            .then(async (res) => {
                const data: any = await res.json();

                if (res.status === 200) {
                    // 토큰을 로컬 스토리지에 저장
                    localStorage.setItem('token', data.token);
                    localStorage.setItem('userinfo', JSON.stringify(data));
                    setAuthState(true);
                    handleClose();
                } else if (res.status === 401) {
                    throw new Error("올바르지 않은 계정이에요")
                } else {
                    throw new Error("예기치 못한 오류에요. 나중에 다시 시도하세요")
                }
            });

        toast.promise(res, {
            loading: "계정을 불러오는 중",
            success: () => {
                const userinfo = JSON.parse(localStorage.getItem('userinfo') as string)
                return `환영합니다! ${userinfo.user.nick_name}님!`
            },
            error: (err) => err.message
        })

    }

    return (
        <div className={`${styles.overlay} ${closing ? styles.overlayHide : ''}`}
             // onClick={(event) => {
             //     event.preventDefault();
             //     event.stopPropagation();
             //     handleClose();
             // }}
        >
            {showSignupDialog && <SignupDialog onClose={() => setShowSignupDialog(false)} />}

            <div className={`${styles.loginDialogContainer} ${closing ? styles.slideDown : ''}`}>
                <div style={{display: 'flex', alignItems: 'center'}}>
                    <p
                        style={{margin: 0, flex: 1}}
                        className={[styles.largeFont, styles.fontBold].join(" ")}
                    >
                        다시 만나서 반가워요!
                    </p>
                    <Image src={icon_close} width={20} height={20} alt={'닫기'} onClick={handleClose} />
                </div>

                <div className={styles.inputGroup}>
                    <input
                        type="email" id="email"
                        name="email" required
                        placeholder="이메일" style={{marginBottom: '10px'}}
                        value={email} onChange={(evt) => setEmail(evt.target.value)}
                    />
                    <input
                        type="password" id="password"
                        name="password" required
                        placeholder="비밀번호"
                        value={password} onChange={(evt) => setPassword(evt.target.value)}
                    />
                </div>

                <div style={{
                    display: 'flex', alignItems: 'center',
                    gap: '15px'
                }}>
                    <button
                        className={styles.button}
                        onClick={() => executeLogin()}
                    >
                        로그인
                    </button>
                    <p
                        style={{margin: 0}}
                        className={[styles.normalFont, styles.grayColor].join(" ")}
                        onClick={() => setShowSignupDialog(!showSignupDialog)}
                    >
                        회원가입
                    </p>
                </div>
            </div>


        </div>
    );
};

export default LoginDialog;
