import React from 'react';
import styles from '../styles/AlertDialog.module.css';
import {Input} from "@nextui-org/input";
import SignupDialog from "@/components/SignupDialog";
import {preventDefaultEvent} from "@material/list/events";
import Image from "next/image";
import icon_close from "../../public/images/icon-close.svg";
import toast, { Toaster } from 'react-hot-toast';
import {exec} from "child_process";
import AlertDialog from "@/components/AlertDialog";


interface SubmissionEvalDialogProps {
    onClose: () => void;
    eventId: string;
    submissionId: string;
}

const SubmissionEvalViewDialog: React.FC<SubmissionEvalDialogProps> = ({onClose, eventId, submissionId}) => {
    const [closing, setClosing] = React.useState(false);
    const [submission, setSubmission] = React.useState<any>({});
    const [openCorrectAlert, setOpenCorrectAlert] = React.useState<boolean>(false);
    const [openInCorrectAlert, setOpenInCorrectAlert] = React.useState<boolean>(false);

    React.useEffect(() => {
        const handleKeyDown = (event: KeyboardEvent) => {
            if (event.key === 'Escape') {
                handleClose();
            }
        };

        window.addEventListener('keydown', handleKeyDown);

        return () => {
            window.removeEventListener('keydown', handleKeyDown);
        };
    }, [onClose]);

    React.useEffect(() => {
        getSubmission(submissionId);
    }, []);

    const handleClose = () => {
        setClosing(true);
        setTimeout(() => {
            onClose();
        }, 200); // 애니메이션 시간 (0.3s) 후 onClose 실행
    };

    const getSubmission = (submission_id: string) => {
        const res = fetch(`https://rd21.cafe24.com:4343/events/submissions/get/${submission_id}/`, {
            method: 'GET',
            headers: {
                'Content-Type': 'application/json'
            },
        })
            .then(async (res) => {
                const data: any = await res.json();

                if (res.status === 200) {
                    setSubmission(data);
                } else if (res.status === 401) {
                    toast.error("조회 권한이 없어요")
                } else {
                    toast.error("예기치 못한 오류에요. 나중에 다시 시도하세요")
                }
            });

    }

    const evalSubmission = (isCorrect: boolean) => {
        const res = fetch(`https://rd21.cafe24.com:4343/events/set/eval/${eventId}/${submissionId}/`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Token ${localStorage.getItem('token')}`
            },
            body: JSON.stringify({
                isCorrect: isCorrect
            })
        })
            .then(async (res) => {
                if (res.status === 200) {
                    if (!isCorrect) {
                        handleClose();
                    }
                } else if (res.status === 401) {
                    throw new Error("조회 권한이 없어요")
                } else {
                    throw new Error("예기치 못한 오류에요. 나중에 다시 시도하세요")
                }
            });

        toast.promise(res, {
            loading: "제출물 평가 중",
            success: '평가 완료!',
            error: (err) => err.message
        })
    }

    const closeEvent = () => {
        // 승자를 정해서 이벤트를 종료시키기
        const res = fetch(`https://rd21.cafe24.com:4343/events/set/winner/${eventId}/${submissionId}/`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Token ${localStorage.getItem('token')}`
            }
        })
            .then(async (res) => {
                if (res.status === 200) {
                    handleClose();
                } else if (res.status === 401) {
                    throw new Error("조회 권한이 없어요")
                } else {
                    throw new Error("예기치 못한 오류에요. 나중에 다시 시도하세요")
                }
            });

        toast.promise(res, {
            loading: "이벤트 종료 중",
            success: '승자가 결정되었으므로 이벤트가 종료되었습니다!',
            error: (err) => err.message
        })
    }

    return (
        <div className={`${styles.overlay} ${closing ? styles.overlayHide : ''}`} style={{zIndex: 60}}
             // onClick={(event) => {
             //     event.preventDefault();
             //     event.stopPropagation();
             //     handleClose();
             // }}
        >
            {
                openCorrectAlert &&
                    <AlertDialog
                        onClose={() => setOpenCorrectAlert(false)}
                        type={'choice'}
                        alertTitle={'정답으로 처리하시겠어요?'}
                        alertDesc={'정답을 맞춘 사용자가 승자로 처리되며, 이벤트는 종료 상태가 됩니다. 계속하시겠어요?'}
                        yesButtonTitle={'정답 처리'}
                        noButtonTitle={'취소'}
                        onClickYes={() => {
                            evalSubmission(true);
                            closeEvent();
                        }}
                    />
            }
            {
                openInCorrectAlert &&
                    <AlertDialog
                        onClose={() => setOpenInCorrectAlert(false)}
                        type={'choice'}
                        alertTitle={'오답으로 처리하시겠어요?'}
                        alertDesc={'해당 제출물은 오답으로 처리됩니다. 계속하시겠어요?'}
                        yesButtonTitle={'오답 처리'}
                        noButtonTitle={'취소'}
                        onClickYes={() => {
                            evalSubmission(false);
                        }}
                    />
            }
            <div className={`${styles.loginDialogContainer} ${closing ? styles.slideDown : ''}`}>
                <div style={{display: 'flex', alignItems: 'center'}}>
                    <p
                        style={{margin: 0, flex: 1}}
                        className={[styles.largeFont, styles.fontBold].join(" ")}
                    >
                        {
                            submission?.author ?
                                submission?.author[0]?.user?.nick_name + '의 제출물'
                                :
                                "제출물"
                        }
                    </p>
                    <Image src={icon_close} width={20} height={20} alt={'닫기'} onClick={handleClose} />
                </div>
                <p
                    className={[styles.extraSmallFont, styles.grayColor].join(" ")}
                    style={{margin: '5px 0px', marginBottom: 0, padding: 0}}
                >
                    제출됨: {new Date(submission?.created_at).toLocaleString()}
                </p>
                <p
                    className={[styles.extraSmallFont, styles.grayColor].join(" ")}
                    style={{margin: '0px 0px', marginTop: 0, marginBottom: '15px'}}
                >
                    심사 상태: {
                        submission?.eval_state === 'waiting' ? '심사 대기 중'
                        :
                            submission?.eval_state === 'correct' ? '정답' : '오답'
                    }
                </p>

                <div style={{
                    width: '100%',
                    flex: 1
                }}>
                    <img src={submission?.image1} alt={'제출물 이미지'}
                         style={{width: '100%', height: '300px', objectFit: 'cover', borderRadius: '15px'}}
                    />
                </div>

                {
                    submission?.eval_state === 'waiting' &&
                        <div style={{
                            display: 'flex', alignItems: 'center',
                            gap: '10px', marginTop: '20px'
                        }}>
                            <button
                                className={styles.noButton}
                                onClick={() => setOpenInCorrectAlert(true)}
                            >
                                오답
                            </button>

                            <button
                                className={styles.yesButton}
                                onClick={() => setOpenCorrectAlert(true)}
                            >
                                정답
                            </button>
                        </div>
                }


            </div>


        </div>
    );
};

export default SubmissionEvalViewDialog;
