import type { AppProps } from 'next/app';
import React, { useState, createContext, useContext, ReactNode } from 'react';
import Toast from "@/components/Toast";
import { NextUIProvider } from '@nextui-org/react';
import '../styles/globals.css';
import '../styles/Toast.module.css';
import {Toaster} from "react-hot-toast";
import { config } from '@fortawesome/fontawesome-svg-core'
import '@fortawesome/fontawesome-svg-core/styles.css'

// font awesome configuration
config.autoAddCss = false

interface ToastContextType {
    showToast: (message: string) => void;
}

const ToastContext = createContext<ToastContextType | undefined>(undefined);

export const useToast = () => {
    const context = useContext(ToastContext);
    if (!context) {
        throw new Error('useToast must be used within a ToastProvider');
    }
    return context;
};

const ToastProvider: React.FC<{ children: ReactNode }> = ({ children }) => {
    const [toastMessage, setToastMessage] = useState<string | null>(null);

    const showToast = (message: string) => {
        setToastMessage(message);
    };

    const handleClose = () => {
        setToastMessage(null);
    };

    return (
        <ToastContext.Provider value={{ showToast }}>
            {children}
            <div className="toast-container">
                {toastMessage && <Toast message={toastMessage} onClose={handleClose} />}
            </div>
        </ToastContext.Provider>
    );
};

const setDocumentHeight = () => {
    const doc = document.documentElement;
    doc.style.setProperty('--doc-height', `${window.innerHeight}px`);
};

function MyApp({ Component, pageProps }: AppProps) {
    React.useEffect(() => {
        setDocumentHeight();
        window.addEventListener('resize', setDocumentHeight);

        return () => {
            window.removeEventListener('resize', setDocumentHeight);
        };
    }, []);

    return (
        <ToastProvider>
            <Toaster />
            <NextUIProvider>
                <Component {...pageProps} />
            </NextUIProvider>
        </ToastProvider>

    );
}

export default MyApp;